# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import copy
import unittest

from opentelemetry.sdk.util import BoundedList


# pylint: disable=unsubscriptable-object
class TestBoundedList(unittest.TestCase):
    base = [52, 36, 53, 29, 54, 99, 56, 48, 22, 35, 21, 65, 10, 95, 42, 60]

    def test_raises(self):
        """Test corner cases

        - negative list size
        - access out of range indexes
        """
        with self.assertRaises(ValueError):
            BoundedList(-1)

        blist = BoundedList(4)
        blist.append(37)
        blist.append(13)

        with self.assertRaises(IndexError):
            _ = blist[2]

        with self.assertRaises(IndexError):
            _ = blist[4]

        with self.assertRaises(IndexError):
            _ = blist[-3]

    def test_from_seq(self):
        list_len = len(self.base)
        base_copy = list(self.base)
        blist = BoundedList.from_seq(list_len, base_copy)

        self.assertEqual(len(blist), list_len)

        # modify base_copy and test that blist is not changed
        for idx in range(list_len):
            base_copy[idx] = idx * base_copy[idx]

        for idx in range(list_len):
            self.assertEqual(blist[idx], self.base[idx])

        # test that iter yields the correct number of elements
        self.assertEqual(len(tuple(blist)), list_len)

        # sequence too big
        blist = BoundedList.from_seq(list_len // 2, base_copy)
        self.assertEqual(len(blist), list_len // 2)
        self.assertEqual(blist.dropped, list_len - (list_len // 2))

    def test_append_no_drop(self):
        """Append max capacity elements to the list without dropping elements."""
        # create empty list
        list_len = len(self.base)
        blist = BoundedList(list_len)
        self.assertEqual(len(blist), 0)

        # fill list
        for item in self.base:
            blist.append(item)

        self.assertEqual(len(blist), list_len)
        self.assertEqual(blist.dropped, 0)

        for idx in range(list_len):
            self.assertEqual(blist[idx], self.base[idx])

        # test __iter__ in BoundedList
        for idx, val in enumerate(blist):
            self.assertEqual(val, self.base[idx])

    def test_append_drop(self):
        """Append more than max capacity elements and test that oldest ones are dropped."""
        list_len = len(self.base)
        # create full BoundedList
        blist = BoundedList.from_seq(list_len, self.base)

        # try to append more items
        for val in self.base:
            # should drop the element without raising exceptions
            blist.append(2 * val)

        self.assertEqual(len(blist), list_len)
        self.assertEqual(blist.dropped, list_len)

        # test that new elements are in the list
        for idx in range(list_len):
            self.assertEqual(blist[idx], 2 * self.base[idx])

    def test_extend_no_drop(self):
        # create empty list
        list_len = len(self.base)
        blist = BoundedList(list_len)
        self.assertEqual(len(blist), 0)

        # fill list
        blist.extend(self.base)

        self.assertEqual(len(blist), list_len)
        self.assertEqual(blist.dropped, 0)

        for idx in range(list_len):
            self.assertEqual(blist[idx], self.base[idx])

        # test __iter__ in BoundedList
        for idx, val in enumerate(blist):
            self.assertEqual(val, self.base[idx])

    def test_extend_drop(self):
        list_len = len(self.base)
        # create full BoundedList
        blist = BoundedList.from_seq(list_len, self.base)
        other_list = [13, 37, 51, 91]

        # try to extend with more elements
        blist.extend(other_list)

        self.assertEqual(len(blist), list_len)
        self.assertEqual(blist.dropped, len(other_list))

    def test_no_limit(self):
        blist = BoundedList(maxlen=None)
        for num in range(100):
            blist.append(num)

        for num in range(100):
            self.assertEqual(blist[num], num)

    # pylint: disable=protected-access
    def test_deepcopy(self):
        blist = BoundedList(maxlen=10)
        blist.append(1)
        blist.append([2, 3])
        blist.dropped = 5

        blist_copy = copy.deepcopy(blist)

        self.assertIsNot(blist, blist_copy)
        self.assertIsNot(blist._dq, blist_copy._dq)
        self.assertIsNot(blist._lock, blist_copy._lock)
        self.assertEqual(list(blist), list(blist_copy))
        self.assertEqual(blist.dropped, blist_copy.dropped)
        self.assertEqual(blist._dq.maxlen, blist_copy._dq.maxlen)
        self.assertIsNot(blist[1], blist_copy[1])
        self.assertEqual(blist[1], blist_copy[1])



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0
# pylint: disable=import-error

import unittest
from unittest.mock import patch

from opentelemetry.exporter.otlp.common._aggregation import (
    _get_aggregation,
    _get_temporality,
)
from opentelemetry.sdk.environment_variables import (
    OTEL_EXPORTER_OTLP_METRICS_DEFAULT_HISTOGRAM_AGGREGATION,
    OTEL_EXPORTER_OTLP_METRICS_TEMPORALITY_PREFERENCE,
)
from opentelemetry.sdk.metrics import (
    Counter,
    Histogram,
    ObservableCounter,
    ObservableGauge,
    ObservableUpDownCounter,
    UpDownCounter,
)
from opentelemetry.sdk.metrics.export import AggregationTemporality
from opentelemetry.sdk.metrics.view import (
    ExplicitBucketHistogramAggregation,
    ExponentialBucketHistogramAggregation,
)

_AGGREGATION_LOGGER_NAME = "opentelemetry.exporter.otlp.common._aggregation"


class TestGetTemporality(unittest.TestCase):
    def test_temporality_default_is_cumulative(self):
        result = _get_temporality(None)
        for instrument_class in (
            Counter,
            UpDownCounter,
            Histogram,
            ObservableCounter,
            ObservableUpDownCounter,
            ObservableGauge,
        ):
            with self.subTest(instrument=instrument_class.__name__):
                self.assertEqual(
                    result[instrument_class],
                    AggregationTemporality.CUMULATIVE,
                )

    def test_temporality_delta_env(self):
        delta_cases = {
            Counter: AggregationTemporality.DELTA,
            UpDownCounter: AggregationTemporality.CUMULATIVE,
            Histogram: AggregationTemporality.DELTA,
            ObservableCounter: AggregationTemporality.DELTA,
            ObservableUpDownCounter: AggregationTemporality.CUMULATIVE,
            ObservableGauge: AggregationTemporality.CUMULATIVE,
        }
        with patch.dict(
            "os.environ",
            {OTEL_EXPORTER_OTLP_METRICS_TEMPORALITY_PREFERENCE: "DELTA"},
        ):
            result = _get_temporality(None)
        for instrument_class, expected in delta_cases.items():
            with self.subTest(instrument=instrument_class.__name__):
                self.assertEqual(result[instrument_class], expected)

    def test_temporality_lowmemory_env(self):
        lowmemory_cases = {
            Counter: AggregationTemporality.DELTA,
            UpDownCounter: AggregationTemporality.CUMULATIVE,
            Histogram: AggregationTemporality.DELTA,
            ObservableCounter: AggregationTemporality.CUMULATIVE,
            ObservableUpDownCounter: AggregationTemporality.CUMULATIVE,
            ObservableGauge: AggregationTemporality.CUMULATIVE,
        }
        with patch.dict(
            "os.environ",
            {OTEL_EXPORTER_OTLP_METRICS_TEMPORALITY_PREFERENCE: "LOWMEMORY"},
        ):
            result = _get_temporality(None)
        for instrument_class, expected in lowmemory_cases.items():
            with self.subTest(instrument=instrument_class.__name__):
                self.assertEqual(result[instrument_class], expected)

    def test_temporality_invalid_env_logs_warning(self):
        with patch.dict(
            "os.environ",
            {OTEL_EXPORTER_OTLP_METRICS_TEMPORALITY_PREFERENCE: "INVALID"},
        ):
            with self.assertLogs(_AGGREGATION_LOGGER_NAME, level="WARNING"):
                result = _get_temporality(None)
        self.assertEqual(
            result[Counter],
            AggregationTemporality.CUMULATIVE,
        )

    def test_temporality_override_takes_precedence(self):
        with patch.dict(
            "os.environ",
            {OTEL_EXPORTER_OTLP_METRICS_TEMPORALITY_PREFERENCE: "CUMULATIVE"},
        ):
            result = _get_temporality({Counter: AggregationTemporality.DELTA})
        self.assertEqual(result[Counter], AggregationTemporality.DELTA)


class TestGetAggregation(unittest.TestCase):
    def test_aggregation_default_is_explicit_bucket(self):
        result = _get_aggregation(None)
        self.assertIsInstance(
            result[Histogram],
            ExplicitBucketHistogramAggregation,
        )

    def test_aggregation_exponential_env(self):
        with patch.dict(
            "os.environ",
            {
                OTEL_EXPORTER_OTLP_METRICS_DEFAULT_HISTOGRAM_AGGREGATION: "base2_exponential_bucket_histogram"
            },
        ):
            result = _get_aggregation(None)
        self.assertIsInstance(
            result[Histogram],
            ExponentialBucketHistogramAggregation,
        )

    def test_aggregation_invalid_env_logs_warning(self):
        with patch.dict(
            "os.environ",
            {
                OTEL_EXPORTER_OTLP_METRICS_DEFAULT_HISTOGRAM_AGGREGATION: "unknown_aggregation"
            },
        ):
            with self.assertLogs(_AGGREGATION_LOGGER_NAME, level="WARNING"):
                result = _get_aggregation(None)
        self.assertIsInstance(
            result[Histogram],
            ExplicitBucketHistogramAggregation,
        )

    def test_aggregation_override_takes_precedence(self):
        custom_aggregation = ExponentialBucketHistogramAggregation()
        with patch.dict(
            "os.environ",
            {
                OTEL_EXPORTER_OTLP_METRICS_DEFAULT_HISTOGRAM_AGGREGATION: "explicit_bucket_histogram"
            },
        ):
            result = _get_aggregation({Histogram: custom_aggregation})
        self.assertIs(result[Histogram], custom_aggregation)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import time
from threading import Lock

from opentelemetry.metrics import CallbackOptions, Observation
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import (
    MetricExporter,
    MetricExportResult,
    MetricsData,
    PeriodicExportingMetricReader,
)
from opentelemetry.test.concurrency_test import ConcurrencyTestBase


class MaxCountExporter(MetricExporter):
    def __init__(self) -> None:
        super().__init__(None, None)
        self._lock = Lock()

        # the number of threads inside of export()
        self.count_in_export = 0

        # the total count of calls to export()
        self.export_count = 0

        # the maximum number of threads in export() ever
        self.max_count_in_export = 0

    def export(
        self,
        metrics_data: MetricsData,
        timeout_millis: float = 10_000,
        **kwargs,
    ) -> MetricExportResult:
        with self._lock:
            self.export_count += 1
            self.count_in_export += 1

        # yield to other threads
        time.sleep(0)

        with self._lock:
            self.max_count_in_export = max(
                self.max_count_in_export, self.count_in_export
            )
            self.count_in_export -= 1

    def force_flush(self, timeout_millis: float = 10_000) -> bool:
        return True

    def shutdown(self, timeout_millis: float = 30_000, **kwargs) -> None:
        pass


class TestExporterConcurrency(ConcurrencyTestBase):
    """
    Tests the requirement that:

    > `Export` will never be called concurrently for the same exporter instance.  `Export` can
    > be called again only after the current call returns.

    https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/metrics/sdk.md#exportbatch

    This test also tests that a thread that calls the a
    ``MetricReader.collect`` method using an asynchronous instrument is able
    to perform two actions in the same thread lock space (without it being
    interrupted by another thread):

    1. Consume the measurement produced by the callback associated to the
       asynchronous instrument.
    2. Export the measurement mentioned in the step above.
    """

    def test_exporter_not_called_concurrently(self):
        exporter = MaxCountExporter()
        reader = PeriodicExportingMetricReader(
            exporter=exporter,
            export_interval_millis=100_000,
        )
        meter_provider = MeterProvider(metric_readers=[reader])

        counter_cb_counter = 0

        def counter_cb(options: CallbackOptions):
            nonlocal counter_cb_counter
            counter_cb_counter += 1
            yield Observation(2)

        meter_provider.get_meter(__name__).create_observable_counter(
            "testcounter", callbacks=[counter_cb]
        )

        # call collect from a bunch of threads to try and enter export() concurrently
        def test_many_threads():
            reader.collect()

        self.run_with_many_threads(test_many_threads, num_threads=100)

        self.assertEqual(counter_cb_counter, 100)
        # no thread should be in export() now
        self.assertEqual(exporter.count_in_export, 0)
        # should be one call for each thread
        self.assertEqual(exporter.export_count, 100)
        # should never have been more than one concurrent call
        self.assertEqual(exporter.max_count_in_export, 1)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import inspect
import unittest
from dataclasses import dataclass
from types import SimpleNamespace
from typing import Any, ClassVar
from unittest.mock import MagicMock, patch

from opentelemetry.configuration._common import (
    _additional_properties,
    _map_compression,
    _parse_headers,
    _parse_otlp_file_output_stream,
    _resolve_component,
    load_entry_point,
)
from opentelemetry.configuration._exceptions import ConfigurationError
from opentelemetry.configuration.models import (
    ExperimentalResourceDetector,
    LogRecordExporter,
    PushMetricExporter,
    Sampler,
    SpanExporter,
    TextMapPropagator,
)


class TestParseHeaders(unittest.TestCase):
    def test_both_none_returns_none(self):
        self.assertIsNone(_parse_headers(None, None))

    def test_empty_headers_list_returns_empty_dict(self):
        self.assertEqual(_parse_headers(None, ""), {})

    def test_headers_list_single_pair(self):
        self.assertEqual(
            _parse_headers(None, "x-api-key=secret"),
            {"x-api-key": "secret"},
        )

    def test_headers_list_multiple_pairs(self):
        self.assertEqual(
            _parse_headers(None, "x-api-key=secret,env=prod"),
            {"x-api-key": "secret", "env": "prod"},
        )

    def test_headers_list_strips_whitespace(self):
        self.assertEqual(
            _parse_headers(None, " x-api-key = secret , env = prod "),
            {"x-api-key": "secret", "env": "prod"},
        )

    def test_headers_list_value_with_equals(self):
        # value contains '=' — only split on the first one
        self.assertEqual(
            _parse_headers(None, "auth=Bearer tok=en"),
            {"auth": "Bearer tok=en"},
        )

    def test_headers_list_invalid_pair_ignored(self):
        # malformed entry (no '=') should be skipped with a warning
        result = _parse_headers(None, "bad,x-key=val")
        self.assertEqual(result, {"x-key": "val"})

    def test_struct_headers_only(self):
        headers = [
            SimpleNamespace(name="x-api-key", value="secret"),
            SimpleNamespace(name="env", value="prod"),
        ]
        self.assertEqual(
            _parse_headers(headers, None),
            {"x-api-key": "secret", "env": "prod"},
        )

    def test_struct_header_none_value_becomes_empty_string(self):
        headers = [SimpleNamespace(name="x-key", value=None)]
        self.assertEqual(_parse_headers(headers, None), {"x-key": ""})

    def test_struct_headers_override_headers_list(self):
        # struct takes priority over headers_list for same key
        headers = [SimpleNamespace(name="x-api-key", value="from-struct")]
        self.assertEqual(
            _parse_headers(headers, "x-api-key=from-list,env=prod"),
            {"x-api-key": "from-struct", "env": "prod"},
        )

    def test_both_empty_struct_and_none_list_returns_empty_dict(self):
        self.assertEqual(_parse_headers([], None), {})


class TestLoadEntryPoint(unittest.TestCase):
    def test_returns_loaded_class(self):
        mock_class = MagicMock()
        mock_ep = MagicMock()
        mock_ep.load.return_value = mock_class
        with patch(
            "opentelemetry.configuration._common.entry_points",
            return_value=[mock_ep],
        ):
            result = load_entry_point("some_group", "some_name")
        self.assertIs(result, mock_class)

    def test_raises_when_not_found(self):
        with patch(
            "opentelemetry.configuration._common.entry_points",
            return_value=[],
        ):
            with self.assertRaises(ConfigurationError) as ctx:
                load_entry_point("some_group", "missing")
        self.assertIn("missing", str(ctx.exception))
        self.assertIn("some_group", str(ctx.exception))

    def test_wraps_load_exception_in_configuration_error(self):
        mock_ep = MagicMock()
        mock_ep.load.side_effect = ImportError("bad import")
        with patch(
            "opentelemetry.configuration._common.entry_points",
            return_value=[mock_ep],
        ):
            with self.assertRaises(ConfigurationError) as ctx:
                load_entry_point("some_group", "some_name")
        self.assertIn("bad import", str(ctx.exception))

    def test_instantiation_error_not_wrapped(self):
        """load_entry_point returns the class; instantiation is the caller's
        responsibility. Errors from calling the returned class are NOT wrapped
        in ConfigurationError — they propagate as-is."""
        mock_class = MagicMock(side_effect=TypeError("bad init"))
        mock_ep = MagicMock()
        mock_ep.load.return_value = mock_class
        with patch(
            "opentelemetry.configuration._common.entry_points",
            return_value=[mock_ep],
        ):
            cls = load_entry_point("some_group", "some_name")
            # load_entry_point itself succeeds
            self.assertIs(cls, mock_class)
            # Calling the returned class raises the original error, not
            # ConfigurationError
            with self.assertRaises(TypeError, msg="bad init"):
                cls()


class _CompressionWithDeflate:
    Gzip = "gzip"
    Deflate = "deflate"


class _CompressionWithoutDeflate:
    Gzip = "gzip"


class TestMapCompression(unittest.TestCase):
    def test_none_returns_none(self):
        self.assertIsNone(_map_compression(None, _CompressionWithDeflate))

    def test_none_string_returns_none(self):
        self.assertIsNone(_map_compression("none", _CompressionWithDeflate))

    def test_gzip_maps_to_gzip(self):
        self.assertEqual(
            _map_compression("gzip", _CompressionWithDeflate), "gzip"
        )

    def test_deflate_maps_when_enabled(self):
        self.assertEqual(
            _map_compression(
                "deflate", _CompressionWithDeflate, allow_deflate=True
            ),
            "deflate",
        )

    def test_deflate_raises_by_default(self):
        with self.assertRaises(ConfigurationError) as ctx:
            _map_compression("deflate", _CompressionWithDeflate)

        self.assertEqual(
            str(ctx.exception),
            "Unsupported compression value 'deflate'. Supported values: "
            "'gzip', 'none'.",
        )

    def test_deflate_raises_when_http_enum_lacks_support(self):
        with self.assertRaises(ConfigurationError) as ctx:
            _map_compression(
                "deflate", _CompressionWithoutDeflate, allow_deflate=True
            )

        self.assertEqual(
            str(ctx.exception),
            "Unsupported compression value 'deflate'. Supported values: "
            "'gzip', 'none'.",
        )

    def test_http_error_message_includes_deflate(self):
        with self.assertRaises(ConfigurationError) as ctx:
            _map_compression(
                "brotli", _CompressionWithDeflate, allow_deflate=True
            )

        self.assertEqual(
            str(ctx.exception),
            "Unsupported compression value 'brotli'. Supported values: "
            "'gzip', 'deflate', 'none'.",
        )


class TestParseOtlpFileOutputStream(unittest.TestCase):
    def test_none_returns_none(self):
        self.assertIsNone(_parse_otlp_file_output_stream(None))

    def test_stdout_returns_none(self):
        self.assertIsNone(_parse_otlp_file_output_stream("stdout"))

    def test_file_uri_returns_path(self):
        self.assertEqual(
            _parse_otlp_file_output_stream("file:///tmp/traces.jsonl"),
            "/tmp/traces.jsonl",
        )

    def test_file_uri_localhost_host_returns_path(self):
        self.assertEqual(
            _parse_otlp_file_output_stream(
                "file://localhost/tmp/traces.jsonl"
            ),
            "/tmp/traces.jsonl",
        )

    def test_file_uri_with_other_host_raises(self):
        with self.assertRaises(ConfigurationError) as ctx:
            _parse_otlp_file_output_stream("file://otherhost/tmp/traces.jsonl")

        self.assertEqual(
            str(ctx.exception),
            "Unsupported output_stream 'file://otherhost/tmp/traces.jsonl' "
            "for otlp_file_development exporter. Supported values: stdout, "
            "file://<path>.",
        )

    def test_file_uri_empty_path_raises(self):
        with self.assertRaises(ConfigurationError):
            _parse_otlp_file_output_stream("file://")

    def test_file_uri_with_query_raises(self):
        with self.assertRaises(ConfigurationError):
            _parse_otlp_file_output_stream("file:///tmp/traces.jsonl?foo=bar")

    def test_file_uri_with_fragment_raises(self):
        with self.assertRaises(ConfigurationError):
            _parse_otlp_file_output_stream("file:///tmp/traces.jsonl#frag")

    def test_unsupported_scheme_raises(self):
        with self.assertRaises(ConfigurationError) as ctx:
            _parse_otlp_file_output_stream("http://example")

        self.assertEqual(
            str(ctx.exception),
            "Unsupported output_stream 'http://example' for "
            "otlp_file_development exporter. Supported values: stdout, "
            "file://<path>.",
        )

    def test_malformed_uri_raises_configuration_error(self):
        with self.assertRaises(ConfigurationError) as ctx:
            _parse_otlp_file_output_stream("file://[::1")

        self.assertIn(
            "Failed to parse output_stream 'file://[::1' for "
            "otlp_file_development exporter",
            str(ctx.exception),
        )

    def test_relative_path_raises(self):
        with self.assertRaises(ConfigurationError) as ctx:
            _parse_otlp_file_output_stream("file:traces.jsonl")

        self.assertEqual(
            str(ctx.exception),
            "Unsupported output_stream 'file:traces.jsonl' for "
            "otlp_file_development exporter. Path must be absolute.",
        )

    def test_trailing_slash_path_raises(self):
        with self.assertRaises(ConfigurationError) as ctx:
            _parse_otlp_file_output_stream("file:///tmp/output/")

        self.assertEqual(
            str(ctx.exception),
            "Unsupported output_stream 'file:///tmp/output/' for "
            "otlp_file_development exporter. Path must be a file, "
            "not a directory.",
        )


class TestAdditionalPropertiesSupport(unittest.TestCase):
    def setUp(self):
        @_additional_properties
        @dataclass
        class _SampleConfig:
            known_field: dict | None = None
            another_field: str | None = None
            additional_properties: ClassVar[dict[str, Any]]

        self.cls = _SampleConfig

    def test_known_fields_work_normally(self):
        obj = self.cls(known_field={}, another_field="val")
        self.assertEqual(obj.known_field, {})
        self.assertEqual(obj.another_field, "val")
        self.assertEqual(obj.additional_properties, {})

    def test_unknown_kwargs_captured_in_additional_properties(self):
        # pylint: disable=unexpected-keyword-arg
        obj = self.cls(my_plugin={"key": "val"})
        self.assertIsNone(obj.known_field)
        self.assertEqual(
            obj.additional_properties, {"my_plugin": {"key": "val"}}
        )

    def test_mixed_known_and_unknown_kwargs(self):
        # pylint: disable=unexpected-keyword-arg
        obj = self.cls(known_field={}, my_plugin={})
        self.assertEqual(obj.known_field, {})
        self.assertEqual(obj.additional_properties, {"my_plugin": {}})

    def test_no_args_creates_empty_additional_properties(self):
        obj = self.cls()
        self.assertIsNone(obj.known_field)
        self.assertEqual(obj.additional_properties, {})

    def test_signature_preserves_known_fields_and_adds_kwargs(self):
        sig = inspect.signature(self.cls)
        param_names = list(sig.parameters.keys())
        self.assertIn("known_field", param_names)
        self.assertIn("another_field", param_names)
        # **kwargs signals that extras are accepted
        kwargs_param = sig.parameters.get("kwargs")
        self.assertIsNotNone(kwargs_param)
        self.assertEqual(kwargs_param.kind, inspect.Parameter.VAR_KEYWORD)


class TestGeneratedModelsHaveAdditionalProperties(unittest.TestCase):
    """Guards against regressions in the custom datamodel-codegen template.

    The codegen/dataclass.jinja2 template conditionally applies the
    @_additional_properties decorator based on the
    additionalPropertiesType template variable. If datamodel-codegen
    changes how it passes this variable, these tests will fail.
    """

    def _assert_supports_additional_properties(self, model_cls):
        # pylint: disable=unexpected-keyword-arg
        obj = model_cls(_test_plugin_key={})
        self.assertTrue(
            hasattr(obj, "additional_properties"),
            f"{model_cls.__name__} missing additional_properties attribute",
        )
        self.assertIn("_test_plugin_key", obj.additional_properties)

    def test_sampler(self):
        self._assert_supports_additional_properties(Sampler)

    def test_span_exporter(self):
        self._assert_supports_additional_properties(SpanExporter)

    def test_text_map_propagator(self):
        self._assert_supports_additional_properties(TextMapPropagator)

    def test_resource_detector(self):
        self._assert_supports_additional_properties(
            ExperimentalResourceDetector
        )

    def test_log_record_exporter(self):
        self._assert_supports_additional_properties(LogRecordExporter)

    def test_push_metric_exporter(self):
        self._assert_supports_additional_properties(PushMetricExporter)


class TestResolveComponent(unittest.TestCase):
    def setUp(self):
        @_additional_properties
        @dataclass
        class _Config:
            builtin_a: dict | None = None
            builtin_b: str | None = None
            additional_properties: ClassVar[dict[str, Any]]

        self.cls = _Config
        self.registry = {
            "builtin_a": lambda v: ("resolved_a", v),
            "builtin_b": lambda v: ("resolved_b", v),
        }

    def test_resolves_builtin_from_registry(self):
        config = self.cls(builtin_a={"key": "val"})
        result = _resolve_component(
            config, self.registry, "test_group", "test component"
        )
        self.assertEqual(result, ("resolved_a", {"key": "val"}))

    def test_resolves_plugin_via_entry_point(self):
        mock_instance = MagicMock()
        mock_class = MagicMock(return_value=mock_instance)
        with patch(
            "opentelemetry.configuration._common.entry_points",
            return_value=[MagicMock(**{"load.return_value": mock_class})],
        ):
            # pylint: disable=unexpected-keyword-arg
            config = self.cls(my_plugin={"opt": "val"})
            result = _resolve_component(
                config, self.registry, "test_group", "test component"
            )
        self.assertIs(result, mock_instance)
        mock_class.assert_called_once_with(opt="val")

    def test_plugin_with_empty_config(self):
        mock_instance = MagicMock()
        mock_class = MagicMock(return_value=mock_instance)
        with patch(
            "opentelemetry.configuration._common.entry_points",
            return_value=[MagicMock(**{"load.return_value": mock_class})],
        ):
            # pylint: disable=unexpected-keyword-arg
            config = self.cls(my_plugin={})
            _resolve_component(
                config, self.registry, "test_group", "test component"
            )
        mock_class.assert_called_once_with()

    def test_no_component_raises_configuration_error(self):
        config = self.cls()
        with self.assertRaises(ConfigurationError):
            _resolve_component(
                config, self.registry, "test_group", "test component"
            )

    def test_plugin_not_found_raises_configuration_error(self):
        with patch(
            "opentelemetry.configuration._common.entry_points",
            return_value=[],
        ):
            # pylint: disable=unexpected-keyword-arg
            config = self.cls(missing_plugin={})
            with self.assertRaises(ConfigurationError):
                _resolve_component(
                    config, self.registry, "test_group", "test component"
                )

    def test_first_registry_match_wins_when_multiple_set(self):
        """When multiple built-in fields are set (which the schema should
        prevent), the first registry match wins."""
        config = self.cls(builtin_a={"a": 1}, builtin_b="b")
        result = _resolve_component(
            config, self.registry, "test_group", "test component"
        )
        # builtin_a comes first in the registry dict
        self.assertEqual(result, ("resolved_a", {"a": 1}))



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0
import json

from opentelemetry import trace as trace_api
from opentelemetry.exporter.zipkin.encoder import (
    _SCOPE_NAME_KEY,
    _SCOPE_VERSION_KEY,
    NAME_KEY,
    VERSION_KEY,
)
from opentelemetry.exporter.zipkin.json.v1 import JsonV1Encoder
from opentelemetry.exporter.zipkin.node_endpoint import NodeEndpoint
from opentelemetry.sdk import trace
from opentelemetry.test.spantestutil import (
    get_span_with_dropped_attributes_events_links,
)
from opentelemetry.trace import TraceFlags, format_span_id, format_trace_id

from .common_tests import (  # pylint: disable=import-error
    TEST_SERVICE_NAME,
    CommonEncoderTestCases,
)


# pylint: disable=protected-access
class TestV1JsonEncoder(CommonEncoderTestCases.CommonJsonEncoderTest):
    @staticmethod
    def get_encoder(*args, **kwargs) -> JsonV1Encoder:
        return JsonV1Encoder(*args, **kwargs)

    def test_encode(self):
        local_endpoint = {"serviceName": TEST_SERVICE_NAME}

        otel_spans = self.get_exhaustive_otel_span_list()
        trace_id = JsonV1Encoder._encode_trace_id(
            otel_spans[0].context.trace_id
        )

        expected_output = [
            {
                "traceId": trace_id,
                "id": JsonV1Encoder._encode_span_id(
                    otel_spans[0].context.span_id
                ),
                "name": otel_spans[0].name,
                "timestamp": otel_spans[0].start_time // 10**3,
                "duration": (otel_spans[0].end_time // 10**3)
                - (otel_spans[0].start_time // 10**3),
                "annotations": [
                    {
                        "timestamp": otel_spans[0].events[0].timestamp
                        // 10**3,
                        "value": json.dumps(
                            {
                                "event0": {
                                    "annotation_bool": True,
                                    "annotation_string": "annotation_test",
                                    "key_float": 0.3,
                                }
                            },
                            sort_keys=True,
                        ),
                        "endpoint": local_endpoint,
                    }
                ],
                "binaryAnnotations": [
                    {
                        "key": "key_bool",
                        "value": "false",
                        "endpoint": local_endpoint,
                    },
                    {
                        "key": "key_string",
                        "value": "hello_world",
                        "endpoint": local_endpoint,
                    },
                    {
                        "key": "key_float",
                        "value": "111.22",
                        "endpoint": local_endpoint,
                    },
                    {
                        "key": "otel.status_code",
                        "value": "OK",
                        "endpoint": local_endpoint,
                    },
                ],
                "debug": True,
                "parentId": JsonV1Encoder._encode_span_id(
                    otel_spans[0].parent.span_id
                ),
            },
            {
                "traceId": trace_id,
                "id": JsonV1Encoder._encode_span_id(
                    otel_spans[1].context.span_id
                ),
                "name": otel_spans[1].name,
                "timestamp": otel_spans[1].start_time // 10**3,
                "duration": (otel_spans[1].end_time // 10**3)
                - (otel_spans[1].start_time // 10**3),
                "binaryAnnotations": [
                    {
                        "key": "key_resource",
                        "value": "some_resource",
                        "endpoint": local_endpoint,
                    },
                    {
                        "key": "otel.status_code",
                        "value": "ERROR",
                        "endpoint": local_endpoint,
                    },
                    {
                        "key": "error",
                        "value": "Example description",
                        "endpoint": local_endpoint,
                    },
                ],
            },
            {
                "traceId": trace_id,
                "id": JsonV1Encoder._encode_span_id(
                    otel_spans[2].context.span_id
                ),
                "name": otel_spans[2].name,
                "timestamp": otel_spans[2].start_time // 10**3,
                "duration": (otel_spans[2].end_time // 10**3)
                - (otel_spans[2].start_time // 10**3),
                "binaryAnnotations": [
                    {
                        "key": "key_string",
                        "value": "hello_world",
                        "endpoint": local_endpoint,
                    },
                    {
                        "key": "key_resource",
                        "value": "some_resource",
                        "endpoint": local_endpoint,
                    },
                ],
            },
            {
                "traceId": trace_id,
                "id": JsonV1Encoder._encode_span_id(
                    otel_spans[3].context.span_id
                ),
                "name": otel_spans[3].name,
                "timestamp": otel_spans[3].start_time // 10**3,
                "duration": (otel_spans[3].end_time // 10**3)
                - (otel_spans[3].start_time // 10**3),
                "binaryAnnotations": [
                    {
                        "key": NAME_KEY,
                        "value": "name",
                        "endpoint": local_endpoint,
                    },
                    {
                        "key": VERSION_KEY,
                        "value": "version",
                        "endpoint": local_endpoint,
                    },
                    {
                        "key": _SCOPE_NAME_KEY,
                        "value": "name",
                        "endpoint": local_endpoint,
                    },
                    {
                        "key": _SCOPE_VERSION_KEY,
                        "value": "version",
                        "endpoint": local_endpoint,
                    },
                ],
            },
        ]

        self.assert_equal_encoded_spans(
            json.dumps(expected_output),
            JsonV1Encoder().serialize(otel_spans, NodeEndpoint()),
        )

    def test_encode_id_zero_padding(self):
        trace_id = 0x0E0C63257DE34C926F9EFCD03927272E
        span_id = 0x04BF92DEEFC58C92
        parent_id = 0x0AAAAAAAAAAAAAAA
        start_time = 683647322 * 10**9  # in ns
        duration = 50 * 10**6
        end_time = start_time + duration

        otel_span = trace._Span(
            name=TEST_SERVICE_NAME,
            context=trace_api.SpanContext(
                trace_id,
                span_id,
                is_remote=False,
                trace_flags=TraceFlags(TraceFlags.SAMPLED),
            ),
            parent=trace_api.SpanContext(trace_id, parent_id, is_remote=False),
            resource=trace.Resource({}),
        )
        otel_span.start(start_time=start_time)
        otel_span.end(end_time=end_time)

        expected_output = [
            {
                "traceId": format_trace_id(trace_id),
                "id": format_span_id(span_id),
                "name": TEST_SERVICE_NAME,
                "timestamp": JsonV1Encoder._nsec_to_usec_round(start_time),
                "duration": JsonV1Encoder._nsec_to_usec_round(duration),
                "debug": True,
                "parentId": format_span_id(parent_id),
            }
        ]

        self.assertEqual(
            json.dumps(expected_output),
            JsonV1Encoder().serialize([otel_span], NodeEndpoint()),
        )

    def _test_encode_max_tag_length(self, max_tag_value_length: int):
        otel_span, expected_tag_output = self.get_data_for_max_tag_length_test(
            max_tag_value_length
        )
        service_name = otel_span.name

        binary_annotations = []
        for tag_key, tag_expected_value in expected_tag_output.items():
            binary_annotations.append(
                {
                    "key": tag_key,
                    "value": tag_expected_value,
                    "endpoint": {"serviceName": service_name},
                }
            )

        expected_output = [
            {
                "traceId": JsonV1Encoder._encode_trace_id(
                    otel_span.context.trace_id
                ),
                "id": JsonV1Encoder._encode_span_id(otel_span.context.span_id),
                "name": service_name,
                "timestamp": JsonV1Encoder._nsec_to_usec_round(
                    otel_span.start_time
                ),
                "duration": JsonV1Encoder._nsec_to_usec_round(
                    otel_span.end_time - otel_span.start_time
                ),
                "binaryAnnotations": binary_annotations,
                "debug": True,
            }
        ]

        self.assert_equal_encoded_spans(
            json.dumps(expected_output),
            JsonV1Encoder(max_tag_value_length).serialize(
                [otel_span], NodeEndpoint()
            ),
        )

    def test_dropped_span_attributes(self):
        otel_span = get_span_with_dropped_attributes_events_links()
        annotations = JsonV1Encoder()._encode_span(otel_span, "test")[
            "binaryAnnotations"
        ]
        annotations = {
            annotation["key"]: annotation["value"]
            for annotation in annotations
        }
        self.assertEqual("1", annotations["otel.dropped_links_count"])
        self.assertEqual("2", annotations["otel.dropped_attributes_count"])
        self.assertEqual("3", annotations["otel.dropped_events_count"])



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from re import UNICODE, compile

_SANITIZE_NAME_RE = compile(r"[^a-zA-Z0-9:]+", UNICODE)
# Same as name, but doesn't allow ":"
_SANITIZE_ATTRIBUTE_KEY_RE = compile(r"[^a-zA-Z0-9]+", UNICODE)

# UCUM style annotations which are text enclosed in curly braces https://ucum.org/ucum#para-6.
# This regex is more permissive than UCUM allows and matches any character within curly braces.
_UNIT_ANNOTATION = compile(r"{.*}")

# Remaps common UCUM and SI units to prometheus conventions. Copied from
# https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/v0.101.0/pkg/translator/prometheus/normalize_name.go#L19
# See specification:
# https://github.com/open-telemetry/opentelemetry-specification/blob/v1.33.0/specification/compatibility/prometheus_and_openmetrics.md#metric-metadata-1
_UNIT_MAPPINGS = {
    # Time
    "d": "days",
    "h": "hours",
    "min": "minutes",
    "s": "seconds",
    "ms": "milliseconds",
    "us": "microseconds",
    "ns": "nanoseconds",
    # Bytes
    "By": "bytes",
    "KiBy": "kibibytes",
    "MiBy": "mebibytes",
    "GiBy": "gibibytes",
    "TiBy": "tibibytes",
    "KBy": "kilobytes",
    "MBy": "megabytes",
    "GBy": "gigabytes",
    "TBy": "terabytes",
    # SI
    "m": "meters",
    "V": "volts",
    "A": "amperes",
    "J": "joules",
    "W": "watts",
    "g": "grams",
    # Misc
    "Cel": "celsius",
    "Hz": "hertz",
    # TODO(https://github.com/open-telemetry/opentelemetry-specification/issues/4058): the
    # specification says to normalize "1" to ratio but that may change. Update this mapping or
    # remove TODO once a decision is made.
    "1": "",
    "%": "percent",
}
# Similar to _UNIT_MAPPINGS, but for "per" unit denominator.
# Example: s => per second (singular)
# Copied from https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/80317ce83ed87a2dff0c316bb939afbfaa823d5e/pkg/translator/prometheus/normalize_name.go#L58
_PER_UNIT_MAPPINGS = {
    "s": "second",
    "m": "minute",
    "h": "hour",
    "d": "day",
    "w": "week",
    "mo": "month",
    "y": "year",
}


def sanitize_full_name(name: str) -> str:
    """sanitize the given metric name according to Prometheus rule, including sanitizing
    leading digits

    https://github.com/open-telemetry/opentelemetry-specification/blob/v1.33.0/specification/compatibility/prometheus_and_openmetrics.md#metric-metadata-1
    """
    # Leading number special case
    if name and name[0].isdigit():
        name = "_" + name[1:]
    return _sanitize_name(name)


def _sanitize_name(name: str) -> str:
    """sanitize the given metric name according to Prometheus rule, but does not handle
    sanitizing a leading digit."""
    return _SANITIZE_NAME_RE.sub("_", name)


def sanitize_attribute(key: str) -> str:
    """sanitize the given metric attribute key according to Prometheus rule.

    https://github.com/open-telemetry/opentelemetry-specification/blob/v1.33.0/specification/compatibility/prometheus_and_openmetrics.md#metric-attributes
    """
    # Leading number special case
    if key and key[0].isdigit():
        key = "_" + key[1:]
    return _SANITIZE_ATTRIBUTE_KEY_RE.sub("_", key)


def map_unit(unit: str) -> str:
    """Maps unit to common prometheus metric names if available and sanitizes any invalid
    characters

    See:
    - https://github.com/open-telemetry/opentelemetry-specification/blob/v1.33.0/specification/compatibility/prometheus_and_openmetrics.md#metric-metadata-1
    - https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/v0.101.0/pkg/translator/prometheus/normalize_name.go#L108
    """
    # remove curly brace unit annotations
    unit = _UNIT_ANNOTATION.sub("", unit)

    if unit in _UNIT_MAPPINGS:
        return _UNIT_MAPPINGS[unit]

    # replace "/" with "per" units like m/s -> meters_per_second
    ratio_unit_subparts = unit.split("/", maxsplit=1)
    if len(ratio_unit_subparts) == 2:
        bottom = _sanitize_name(ratio_unit_subparts[1])
        if bottom:
            top = _sanitize_name(ratio_unit_subparts[0])
            top = _UNIT_MAPPINGS.get(top, top)
            bottom = _PER_UNIT_MAPPINGS.get(bottom, bottom)
            return f"{top}_per_{bottom}" if top else f"per_{bottom}"

    return (
        # since units end up as a metric name suffix, they must be sanitized
        _sanitize_name(unit)
        # strip surrounding "_" chars since it will lead to consecutive underscores in the
        # metric name
        .strip("_")
    )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import functools
from dataclasses import dataclass, field
from typing import TYPE_CHECKING

# pylint: disable-next=import-error
from opentelemetry.exporter.http.transport._base import (
    BaseHTTPResult,
    BaseHTTPTransport,
)

if TYPE_CHECKING:
    from collections.abc import Mapping
    from typing import Any

    import requests
    from requests import Response


@functools.cache
def _get_connection_error_types() -> tuple[type[Exception], ...]:
    # pylint: disable-next=import-outside-toplevel
    import requests.exceptions  # noqa: PLC0415

    return (
        requests.exceptions.ConnectionError,
        requests.exceptions.ConnectTimeout,
        requests.exceptions.ReadTimeout,
        requests.exceptions.Timeout,
        requests.exceptions.SSLError,
        requests.exceptions.ProxyError,
    )


@dataclass(frozen=True, slots=True)
class RequestsHTTPResult(BaseHTTPResult):
    response: Response | None = field(default=None, hash=False, compare=False)

    def content(self) -> bytes:
        if self.response is None:
            return b""
        return self.response.content or b""

    def text(self) -> str:
        if self.response is None:
            return ""
        return self.response.text or ""

    def json(self) -> Any:
        if self.response is None:
            raise ValueError("No response available.")
        return self.response.json()

    def headers(self) -> Mapping[str, str]:
        if self.response is None:
            return {}
        return self.response.headers


class RequestsHTTPTransport(BaseHTTPTransport):
    def __init__(
        self,
        *,
        verify: bool | str = True,
        cert: str | tuple[str, str] | None = None,
        session: requests.Session | None = None,
        **kwargs: Any,
    ) -> None:
        # pylint: disable-next=import-outside-toplevel
        import requests  # noqa: PLC0415

        self._session = session if session is not None else requests.Session()
        self._session.verify = verify
        if cert is not None:
            self._session.cert = cert

    def request(
        self,
        method: str,
        url: str,
        *,
        headers: dict[str, str] | None = None,
        timeout: float | None = None,
        data: bytes | None = None,
    ) -> BaseHTTPResult:
        try:
            response = self._session.request(
                method=method,
                url=url,
                headers=headers,
                data=data,
                timeout=timeout,
                allow_redirects=False,
            )
        # pylint: disable-next=broad-exception-caught
        except Exception as error:
            # pylint: disable-next=unexpected-keyword-arg
            return RequestsHTTPResult(error=error)

        # pylint: disable-next=unexpected-keyword-arg
        return RequestsHTTPResult(
            status_code=response.status_code,
            reason=response.reason,
            response=response,
        )

    # pylint: disable-next=no-self-use
    def is_connection_error(self, exception: Exception | None) -> bool:
        return isinstance(exception, _get_connection_error_types())

    def close(self) -> None:
        self._session.close()



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# Tests access private members of SDK classes to assert correct configuration.
# pylint: disable=protected-access

import unittest
from dataclasses import dataclass
from typing import Any, ClassVar

from opentelemetry.configuration._common import _additional_properties
from opentelemetry.configuration._conversion import _dict_to_dataclass
from opentelemetry.configuration.models import ExemplarFilter


@dataclass
class _Inner:
    value: int | None = None


@dataclass
class _Middle:
    inner: _Inner | None = None
    items: list[_Inner] | None = None


@dataclass
class _Outer:
    middle: _Middle | None = None
    name: str | None = None


@_additional_properties
@dataclass
class _WithExtras:
    known: str | None = None
    additional_properties: ClassVar[dict[str, Any]]


@dataclass
class _WithEnum:
    filter: ExemplarFilter | None = None


class TestDictToDataclass(unittest.TestCase):
    def test_raises_on_non_dataclass(self):
        # _dict_to_dataclass is internal and assumes cls is a dataclass.
        with self.assertRaises(TypeError) as ctx:
            _dict_to_dataclass({"x": 1}, dict)
        self.assertIn("not a dataclass", str(ctx.exception))

    def test_converts_flat_dict(self):
        result = _dict_to_dataclass({"value": 42}, _Inner)
        self.assertIsInstance(result, _Inner)
        self.assertEqual(result.value, 42)

    def test_converts_nested_dataclass(self):
        result = _dict_to_dataclass(
            {"middle": {"inner": {"value": 7}}}, _Outer
        )
        self.assertIsInstance(result, _Outer)
        self.assertIsInstance(result.middle, _Middle)
        self.assertIsInstance(result.middle.inner, _Inner)
        self.assertEqual(result.middle.inner.value, 7)

    def test_converts_list_of_dataclasses(self):
        result = _dict_to_dataclass(
            {"middle": {"items": [{"value": 1}, {"value": 2}]}}, _Outer
        )
        self.assertEqual(len(result.middle.items), 2)
        self.assertIsInstance(result.middle.items[0], _Inner)
        self.assertEqual(result.middle.items[0].value, 1)
        self.assertEqual(result.middle.items[1].value, 2)

    def test_none_value_preserved(self):
        result = _dict_to_dataclass({"middle": None, "name": "test"}, _Outer)
        self.assertIsNone(result.middle)
        self.assertEqual(result.name, "test")

    def test_missing_optional_fields_default_to_none(self):
        result = _dict_to_dataclass({}, _Outer)
        self.assertIsNone(result.middle)
        self.assertIsNone(result.name)

    def test_unknown_keys_routed_to_additional_properties(self):
        result = _dict_to_dataclass(
            {"known": "yes", "my_plugin": {"opt": True}}, _WithExtras
        )
        self.assertEqual(result.known, "yes")
        self.assertEqual(
            result.additional_properties, {"my_plugin": {"opt": True}}
        )

    def test_primitive_values_pass_through(self):
        result = _dict_to_dataclass({"name": "hello"}, _Outer)
        self.assertEqual(result.name, "hello")

    def test_empty_list_converted(self):
        result = _dict_to_dataclass({"middle": {"items": []}}, _Outer)
        self.assertEqual(result.middle.items, [])

    def test_enum_value_coerced_from_string(self):
        result = _dict_to_dataclass({"filter": "always_on"}, _WithEnum)
        self.assertIs(result.filter, ExemplarFilter.always_on)

    def test_enum_value_already_enum_passes_through(self):
        result = _dict_to_dataclass(
            {"filter": ExemplarFilter.trace_based}, _WithEnum
        )
        self.assertIs(result.filter, ExemplarFilter.trace_based)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import random
import time

from opentelemetry.metrics import get_meter_provider, set_meter_provider
from opentelemetry.sdk.metrics import Counter, MeterProvider
from opentelemetry.sdk.metrics.export import (
    ConsoleMetricExporter,
    PeriodicExportingMetricReader,
)
from opentelemetry.sdk.metrics.view import View

# Create a view matching the counter instrument `my.counter`
# and configure the new name `my.counter.total` for the result metrics stream
change_metric_name_view = View(
    instrument_type=Counter,
    instrument_name="my.counter",
    name="my.counter.total",
)

# Use console exporter for the example
exporter = ConsoleMetricExporter()

# Create a metric reader with stdout exporter
reader = PeriodicExportingMetricReader(exporter, export_interval_millis=1_000)
provider = MeterProvider(
    metric_readers=[
        reader,
    ],
    views=[
        change_metric_name_view,
    ],
)
set_meter_provider(provider)

meter = get_meter_provider().get_meter("view-name-change", "0.1.2")

my_counter = meter.create_counter("my.counter")

while 1:
    my_counter.add(random.randint(1, 10))
    time.sleep(random.random())



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from dataclasses import dataclass
from logging import ERROR, WARNING
from unittest import TestCase
from unittest.mock import MagicMock, patch

from opentelemetry.configuration._exceptions import ConfigurationError
from opentelemetry.configuration.instrumentation import (
    configure_instrumentation,
)
from opentelemetry.configuration.models import ExperimentalInstrumentation

_LOAD_EP = "opentelemetry.configuration.instrumentation.load_entry_point"


def _make_instrumentor_class(instance, configuration=None):
    """Return a class mock whose constructor returns ``instance``."""
    cls = MagicMock(return_value=instance)
    cls.configuration = configuration
    instance.is_instrumented_by_opentelemetry = False
    return cls


class TestConfigureInstrumentation(TestCase):
    # pylint: disable=no-self-use
    def test_none_config_is_noop(self):
        configure_instrumentation(None)

    def test_none_python_is_noop(self):
        configure_instrumentation(ExperimentalInstrumentation())

    @patch(_LOAD_EP, side_effect=ConfigurationError("not found"))
    def test_unknown_instrumentor_logs_warning(self, _mock_load):
        with self.assertLogs(
            "opentelemetry.configuration.instrumentation",
            level=WARNING,
        ) as cm:
            configure_instrumentation(
                ExperimentalInstrumentation(python={"unknown_lib": {}})
            )
        self.assertTrue(
            any("unknown_lib" in msg for msg in cm.output),
            f"Expected warning mentioning 'unknown_lib', got: {cm.output}",
        )

    @patch(_LOAD_EP)
    def test_instruments_listed_library_with_no_opts(self, mock_load):
        instrumentor = MagicMock()
        mock_load.return_value = _make_instrumentor_class(instrumentor)

        configure_instrumentation(
            ExperimentalInstrumentation(python={"requests": {}})
        )

        mock_load.assert_called_once_with(
            "opentelemetry_instrumentor", "requests"
        )
        instrumentor.instrument.assert_called_once_with()

    @patch(_LOAD_EP)
    def test_forwards_kwargs_to_instrumentor_without_configuration(
        self, mock_load
    ):
        instrumentor = MagicMock()
        mock_load.return_value = _make_instrumentor_class(instrumentor)

        configure_instrumentation(
            ExperimentalInstrumentation(
                python={"flask": {"excluded_urls": "/healthz", "foo": "bar"}}
            )
        )

        instrumentor.instrument.assert_called_once_with(
            excluded_urls="/healthz", foo="bar"
        )

    @patch(_LOAD_EP)
    def test_enabled_false_skips_instrumentation(self, mock_load):
        instrumentor = MagicMock()
        mock_load.return_value = _make_instrumentor_class(instrumentor)

        configure_instrumentation(
            ExperimentalInstrumentation(
                python={"requests": {"enabled": False}}
            )
        )

        mock_load.assert_not_called()
        instrumentor.instrument.assert_not_called()

    @patch(_LOAD_EP)
    def test_enabled_key_not_forwarded_to_instrumentor(self, mock_load):
        instrumentor = MagicMock()
        mock_load.return_value = _make_instrumentor_class(instrumentor)

        configure_instrumentation(
            ExperimentalInstrumentation(
                python={"flask": {"enabled": True, "excluded_urls": "/ok"}}
            )
        )

        instrumentor.instrument.assert_called_once_with(excluded_urls="/ok")

    @patch(_LOAD_EP)
    def test_multiple_instrumentors_all_called(self, mock_load):
        flask_inst = MagicMock()
        requests_inst = MagicMock()

        def _side_effect(_group, name):
            return _make_instrumentor_class(
                flask_inst if name == "flask" else requests_inst
            )

        mock_load.side_effect = _side_effect

        configure_instrumentation(
            ExperimentalInstrumentation(
                python={"flask": {}, "requests": {"foo": "bar"}}
            )
        )

        flask_inst.instrument.assert_called_once_with()
        requests_inst.instrument.assert_called_once_with(foo="bar")

    @patch(_LOAD_EP)
    def test_instrumentor_exception_does_not_stop_others(self, mock_load):
        broken_inst = MagicMock()
        broken_inst.instrument.side_effect = RuntimeError("boom")
        ok_inst = MagicMock()

        call_count = 0

        def _side_effect(_group, _name):
            nonlocal call_count
            call_count += 1
            return _make_instrumentor_class(
                broken_inst if call_count == 1 else ok_inst
            )

        mock_load.side_effect = _side_effect

        with self.assertLogs(
            "opentelemetry.configuration.instrumentation",
            level=ERROR,
        ):
            configure_instrumentation(
                ExperimentalInstrumentation(python={"broken": {}, "ok": {}})
            )

        ok_inst.instrument.assert_called_once_with()

    @patch(_LOAD_EP)
    def test_skips_already_instrumented(self, mock_load):
        instrumentor = MagicMock()
        mock_load.return_value = _make_instrumentor_class(instrumentor)
        instrumentor.is_instrumented_by_opentelemetry = True

        configure_instrumentation(
            ExperimentalInstrumentation(python={"requests": {}})
        )

        instrumentor.instrument.assert_not_called()

    @patch(_LOAD_EP)
    def test_non_dataclass_configuration_attribute_ignored(self, mock_load):
        instrumentor = MagicMock()
        mock_load.return_value = _make_instrumentor_class(
            instrumentor, configuration="not-a-dataclass"
        )

        configure_instrumentation(
            ExperimentalInstrumentation(python={"requests": {"foo": "bar"}})
        )

        instrumentor.instrument.assert_called_once_with(foo="bar")

    @patch(_LOAD_EP)
    def test_configuration_coerces_opts(self, mock_load):
        @dataclass
        class RequestsConfig:
            excluded_urls: str | None = None
            capture_headers: bool | None = None

        instrumentor = MagicMock()
        mock_load.return_value = _make_instrumentor_class(
            instrumentor, configuration=RequestsConfig
        )

        configure_instrumentation(
            ExperimentalInstrumentation(
                python={
                    "requests": {
                        "excluded_urls": "/health",
                        "capture_headers": True,
                    }
                }
            )
        )

        instrumentor.instrument.assert_called_once_with(
            excluded_urls="/health", capture_headers=True
        )

    @patch(_LOAD_EP)
    def test_configuration_none_fields_not_forwarded(self, mock_load):
        @dataclass
        class FlaskConfig:
            excluded_urls: str | None = None
            propagate_headers: bool | None = None

        instrumentor = MagicMock()
        mock_load.return_value = _make_instrumentor_class(
            instrumentor, configuration=FlaskConfig
        )

        configure_instrumentation(
            ExperimentalInstrumentation(
                python={"flask": {"excluded_urls": "/ok"}}
            )
        )

        # propagate_headers was not set, so it must not appear in the call.
        instrumentor.instrument.assert_called_once_with(excluded_urls="/ok")

    @patch(_LOAD_EP)
    def test_configuration_unknown_field_raises(self, mock_load):
        @dataclass
        class StrictConfig:
            excluded_urls: str | None = None

        instrumentor = MagicMock()
        mock_load.return_value = _make_instrumentor_class(
            instrumentor, configuration=StrictConfig
        )

        with self.assertLogs(
            "opentelemetry.configuration.instrumentation",
            level=ERROR,
        ):
            configure_instrumentation(
                ExperimentalInstrumentation(
                    python={
                        "mylib": {"excluded_urls": "/ok", "typo_field": "bad"}
                    }
                )
            )

        instrumentor.instrument.assert_not_called()



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# oltpcollector_example.py
import os
import subprocess
import sys
import unittest


class TestOTLPCollector(unittest.TestCase):
    def test_otlpcollector(self):
        """Test that OTLP collector example outputs 'Hello world!'"""
        dirpath = os.path.dirname(os.path.realpath(__file__))
        test_script = f"{dirpath}/../otlpcollector_example.py"

        # Run the script with a short timeout since it will retry forever
        with subprocess.Popen(
            [sys.executable, test_script],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
        ) as process:
            # Wait 2 seconds then kill it (enough time to print "Hello world!")
            try:
                stdout, _ = process.communicate(timeout=2)
            except subprocess.TimeoutExpired:
                process.kill()
                stdout, _ = process.communicate()

        # Check that it printed the expected message
        self.assertIn("Hello world!", stdout)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

"""Tests verifying compatibility between opentelemetry-exporter-otlp-json-common
and protobuf's built-in JSON serialization (MessageToDict / ParseDict).

OTLP JSON deviations from standard proto3 JSON mapping:
- traceId/spanId/parentSpanId: hex-encoded (not base64)
- Enum fields: integer values only
- Keys: lowerCamelCase
- 64-bit integers: string-encoded
"""

import base64
import unittest
from typing import Any

from google.protobuf.json_format import MessageToDict, ParseDict

from opentelemetry._logs import SeverityNumber
from opentelemetry.exporter.otlp.json.common._log_encoder import (
    encode_logs as json_encode_logs,
)
from opentelemetry.exporter.otlp.json.common.metrics_encoder import (
    encode_metrics as json_encode_metrics,
)
from opentelemetry.exporter.otlp.json.common.trace_encoder import (
    encode_spans as json_encode_spans,
)
from opentelemetry.exporter.otlp.proto.common._log_encoder import (
    encode_logs as proto_encode_logs,
)
from opentelemetry.exporter.otlp.proto.common.metrics_encoder import (
    encode_metrics as proto_encode_metrics,
)
from opentelemetry.exporter.otlp.proto.common.trace_encoder import (
    encode_spans as proto_encode_spans,
)
from opentelemetry.proto.collector.logs.v1.logs_service_pb2 import (
    ExportLogsServiceRequest as PB2ExportLogsServiceRequest,
)
from opentelemetry.proto.collector.metrics.v1.metrics_service_pb2 import (
    ExportMetricsServiceRequest as PB2ExportMetricsServiceRequest,
)
from opentelemetry.proto.collector.trace.v1.trace_service_pb2 import (
    ExportTraceServiceRequest as PB2ExportTraceServiceRequest,
)
from opentelemetry.sdk.metrics import Exemplar
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import Event, SpanContext
from opentelemetry.sdk.util.instrumentation import InstrumentationScope
from opentelemetry.trace import Link
from tests import (
    BASE_TIME,
    PARENT_SPAN_ID,
    SPAN_ID,
    TIME,
    TRACE_ID,
    make_exponential_histogram,
    make_gauge,
    make_histogram,
    make_log,
    make_log_context,
    make_metrics_data,
    make_span,
    make_sum,
)

_HEX_ID_KEYS = frozenset({"traceId", "spanId", "parentSpanId"})


def _normalize_otlp_json(data: Any) -> Any:
    """Convert proto3 standard JSON (base64 IDs) to OTLP JSON (hex IDs)."""
    if isinstance(data, list):
        return [_normalize_otlp_json(item) for item in data]
    if isinstance(data, dict):
        return {
            key: (
                base64.b64decode(value).hex()
                if key in _HEX_ID_KEYS and isinstance(value, str)
                else _normalize_otlp_json(value)
            )
            for key, value in data.items()
        }
    return data


def _denormalize_otlp_json(data: Any) -> Any:
    """Convert OTLP JSON (hex IDs) back to proto3 standard JSON (base64 IDs)."""
    if isinstance(data, list):
        return [_denormalize_otlp_json(item) for item in data]
    if isinstance(data, dict):
        return {
            key: (
                base64.b64encode(bytes.fromhex(value)).decode("utf-8")
                if key in _HEX_ID_KEYS and isinstance(value, str)
                else _denormalize_otlp_json(value)
            )
            for key, value in data.items()
        }
    return data


def _make_spans():
    parent_ctx = SpanContext(TRACE_ID, PARENT_SPAN_ID, is_remote=True)
    link_ctx = SpanContext(TRACE_ID, 0x2222222222222222, is_remote=False)

    span1 = make_span(
        name="span-with-everything",
        parent=parent_ctx,
        events=(
            Event(
                name="event1",
                timestamp=BASE_TIME + 10 * 10**6,
                attributes={"event_key": "event_val", "event_int": 42},
            ),
        ),
        links=(Link(context=link_ctx, attributes={"link_bool": True}),),
        resource=Resource(
            {"service.name": "test-service"},
            schema_url="resource_schema",
        ),
        instrumentation_scope=InstrumentationScope(
            name="test_lib",
            version="1.0.0",
            schema_url="scope_schema",
        ),
    )

    span2 = make_span(
        name="bare-span",
        span_id=0xBBBBBBBBBBBBBBBB,
        parent=None,
        resource=Resource({"service.name": "test-service"}),
        start_time=BASE_TIME + 100 * 10**6,
        end_time=BASE_TIME + 200 * 10**6,
    )

    return [span1, span2]


def _make_test_metrics_data():
    return make_metrics_data(
        [
            make_sum(
                name="counter",
                value=100,
                attributes={"method": "GET"},
                description="a counter",
                unit="1",
            ),
            make_gauge(
                name="gauge_float",
                value=52.5,
                attributes={"host": "localhost"},
                description="a gauge",
                unit="ms",
            ),
            make_histogram(
                name="histogram",
                attributes={"path": "/api"},
                count=10,
                sum_value=500.0,
                bucket_counts=[2, 5, 3],
                explicit_bounds=[100.0, 250.0],
                min_value=10.0,
                max_value=450.0,
                exemplars=[
                    Exemplar(
                        {"sampled": "true"},
                        298.0,
                        TIME,
                        SPAN_ID,
                        TRACE_ID,
                    ),
                ],
                description="a histogram",
                unit="ms",
            ),
            make_exponential_histogram(
                name="exp_histogram",
                attributes={},
                count=8,
                sum_value=80.0,
                scale=2,
                zero_count=1,
                min_value=1.0,
                max_value=30.0,
                description="exponential",
            ),
        ],
        resource_attrs={"service.name": "metrics-svc"},
        resource_schema_url="resource_schema",
        scope_name="metrics_lib",
        scope_version="2.0",
        scope_schema_url="scope_schema",
    )


def _make_logs():
    ctx = make_log_context()
    return [
        make_log(
            body="something went wrong",
            severity_text="ERROR",
            severity_number=SeverityNumber.ERROR,
            attributes={"error.code": 500, "path": "/api"},
            context=ctx,
            resource=Resource({"service.name": "log-svc"}, "resource_schema"),
            instrumentation_scope=InstrumentationScope(
                "log_lib", "1.0", "scope_schema"
            ),
        ),
        make_log(
            body="healthy",
            timestamp=TIME + 2000,
            observed_timestamp=TIME + 3000,
            resource=Resource({"service.name": "log-svc"}),
            instrumentation_scope=InstrumentationScope("log_lib", "1.0"),
        ),
    ]


class TestProtoJsonDictCompatibility(unittest.TestCase):
    """Compare .to_dict() from JSON encoder with MessageToDict from proto
    encoder (after normalizing IDs)."""

    def test_trace_dict_compatibility(self):
        spans = _make_spans()
        json_dict = json_encode_spans(spans).to_dict()
        proto_dict = MessageToDict(
            proto_encode_spans(spans),
            preserving_proto_field_name=False,
            use_integers_for_enums=True,
        )
        self.assertEqual(json_dict, _normalize_otlp_json(proto_dict))

    def test_metrics_dict_compatibility(self):
        data = _make_test_metrics_data()
        json_dict = json_encode_metrics(data).to_dict()
        proto_dict = MessageToDict(
            proto_encode_metrics(data),
            preserving_proto_field_name=False,
            use_integers_for_enums=True,
        )
        self.assertEqual(json_dict, _normalize_otlp_json(proto_dict))

    def test_log_dict_compatibility(self):
        logs = _make_logs()
        json_dict = json_encode_logs(logs).to_dict()
        proto_dict = MessageToDict(
            proto_encode_logs(logs),
            preserving_proto_field_name=False,
            use_integers_for_enums=True,
        )
        self.assertEqual(json_dict, _normalize_otlp_json(proto_dict))


class TestProtoJsonParseCompatibility(unittest.TestCase):
    """Encode SDK objects to dict via JSON encoder, denormalize (hex→base64
    IDs), parse into protobuf via ParseDict, compare with proto encoder."""

    def test_trace_parse_compatibility(self):
        spans = _make_spans()
        json_dict = json_encode_spans(spans).to_dict()
        proto_expected = proto_encode_spans(spans)

        denormalized = _denormalize_otlp_json(json_dict)
        proto_parsed = ParseDict(denormalized, PB2ExportTraceServiceRequest())
        self.assertEqual(proto_parsed, proto_expected)

    def test_metrics_parse_compatibility(self):
        data = _make_test_metrics_data()
        json_dict = json_encode_metrics(data).to_dict()
        proto_expected = proto_encode_metrics(data)

        denormalized = _denormalize_otlp_json(json_dict)
        proto_parsed = ParseDict(
            denormalized, PB2ExportMetricsServiceRequest()
        )
        self.assertEqual(proto_parsed, proto_expected)

    def test_log_parse_compatibility(self):
        logs = _make_logs()
        json_dict = json_encode_logs(logs).to_dict()
        proto_expected = proto_encode_logs(logs)

        denormalized = _denormalize_otlp_json(json_dict)
        proto_parsed = ParseDict(denormalized, PB2ExportLogsServiceRequest())
        self.assertEqual(proto_parsed, proto_expected)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import asyncio

# pylint: disable=import-error
from ..otel_ot_shim_tracer import MockTracer
from ..testcase import OpenTelemetryTestCase
from ..utils import stop_loop_when


class TestAsyncio(OpenTelemetryTestCase):
    def setUp(self):
        self.tracer = MockTracer()
        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)

    def tearDown(self):
        self.loop.close()
        super().tearDown()

    def test_main(self):
        # Start an isolated task and query for its result -and finish it-
        # in another task/thread
        span = self.tracer.start_span("initial")
        self.submit_another_task(span)

        stop_loop_when(
            self.loop,
            lambda: len(self.tracer.finished_spans()) >= 3,
            timeout=5.0,
        )
        self.loop.run_forever()

        spans = self.tracer.finished_spans()
        self.assertEqual(len(spans), 3)
        self.assertNamesEqual(spans, ["initial", "subtask", "task"])

        # task/subtask are part of the same trace,
        # and subtask is a child of task
        self.assertSameTrace(spans[1], spans[2])
        self.assertIsChildOf(spans[1], spans[2])

        # initial task is not related in any way to those two tasks
        self.assertNotSameTrace(spans[0], spans[1])
        self.assertEqual(spans[0].parent, None)

    async def task(self, span):
        # Create a new Span for this task
        with self.tracer.start_active_span("task"):
            with self.tracer.scope_manager.activate(span, True):
                # Simulate work strictly related to the initial Span
                pass

            # Use the task span as parent of a new subtask
            with self.tracer.start_active_span("subtask"):
                pass

    def submit_another_task(self, span):
        self.loop.create_task(self.task(span))



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# AUTO-GENERATED from "opentelemetry/proto/collector/profiles/v1development/profiles_service.proto"
# DO NOT EDIT MANUALLY

from __future__ import annotations

import builtins
import dataclasses
import functools
import typing

_dataclass = functools.partial(dataclasses.dataclass, slots=True)

import opentelemetry.proto_json._json_codec
import opentelemetry.proto_json.profiles.v1development.profiles


@typing.final
@_dataclass
class ExportProfilesServiceRequest(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ExportProfilesServiceRequest
    """

    resource_profiles: builtins.list[opentelemetry.proto_json.profiles.v1development.profiles.ResourceProfiles] = dataclasses.field(default_factory=builtins.list)
    dictionary: opentelemetry.proto_json.profiles.v1development.profiles.ProfilesDictionary | None = None

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.resource_profiles:
            _result["resourceProfiles"] = opentelemetry.proto_json._json_codec.encode_repeated(self.resource_profiles, lambda _v: _v.to_dict())
        if self.dictionary:
            _result["dictionary"] = self.dictionary.to_dict()
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ExportProfilesServiceRequest":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ExportProfilesServiceRequest instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("resourceProfiles")) is not None:
            _args["resource_profiles"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.profiles.v1development.profiles.ResourceProfiles.from_dict(_v), "resource_profiles")
        if (_value := data.get("dictionary")) is not None:
            _args["dictionary"] = opentelemetry.proto_json.profiles.v1development.profiles.ProfilesDictionary.from_dict(_value)

        return cls(**_args)


@typing.final
@_dataclass
class ExportProfilesServiceResponse(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ExportProfilesServiceResponse
    """

    partial_success: ExportProfilesPartialSuccess | None = None

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.partial_success:
            _result["partialSuccess"] = self.partial_success.to_dict()
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ExportProfilesServiceResponse":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ExportProfilesServiceResponse instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("partialSuccess")) is not None:
            _args["partial_success"] = ExportProfilesPartialSuccess.from_dict(_value)

        return cls(**_args)


@typing.final
@_dataclass
class ExportProfilesPartialSuccess(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ExportProfilesPartialSuccess
    """

    rejected_profiles: builtins.int | None = 0
    error_message: builtins.str | None = ""

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.rejected_profiles:
            _result["rejectedProfiles"] = opentelemetry.proto_json._json_codec.encode_int64(self.rejected_profiles)
        if self.error_message:
            _result["errorMessage"] = self.error_message
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ExportProfilesPartialSuccess":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ExportProfilesPartialSuccess instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("rejectedProfiles")) is not None:
            _args["rejected_profiles"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "rejected_profiles")
        if (_value := data.get("errorMessage")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "error_message")
            _args["error_message"] = _value

        return cls(**_args)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum
from typing import Final

from typing_extensions import deprecated

TELEMETRY_DISTRO_NAME: Final = "telemetry.distro.name"
"""
Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TELEMETRY_DISTRO_NAME`.
"""

TELEMETRY_DISTRO_VERSION: Final = "telemetry.distro.version"
"""
Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TELEMETRY_DISTRO_VERSION`.
"""

TELEMETRY_SDK_LANGUAGE: Final = "telemetry.sdk.language"
"""
Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TELEMETRY_SDK_LANGUAGE`.
"""

TELEMETRY_SDK_NAME: Final = "telemetry.sdk.name"
"""
Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TELEMETRY_SDK_NAME`.
"""

TELEMETRY_SDK_VERSION: Final = "telemetry.sdk.version"
"""
Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TELEMETRY_SDK_VERSION`.
"""


@deprecated(
    "Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues`."
)
class TelemetrySdkLanguageValues(Enum):
    CPP = "cpp"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.CPP`."""
    DOTNET = "dotnet"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.DOTNET`."""
    ERLANG = "erlang"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.ERLANG`."""
    GO = "go"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.GO`."""
    JAVA = "java"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.JAVA`."""
    KOTLIN = "kotlin"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.KOTLIN`."""
    NODEJS = "nodejs"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.NODEJS`."""
    PHP = "php"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.PHP`."""
    PYTHON = "python"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.PYTHON`."""
    RUBY = "ruby"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.RUBY`."""
    RUST = "rust"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.RUST`."""
    SWIFT = "swift"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.SWIFT`."""
    WEBJS = "webjs"
    """Deprecated in favor of stable :py:const:`opentelemetry.semconv.attributes.telemetry_attributes.TelemetrySdkLanguageValues.WEBJS`."""



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# TODO: make pylint use 3p opentracing module for type inference
# pylint:disable=no-member

import time
import traceback
from unittest import TestCase
from unittest.mock import Mock

import opentracing

from opentelemetry import trace
from opentelemetry.propagate import get_global_textmap, set_global_textmap
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.shim.opentracing_shim import (
    SpanContextShim,
    SpanShim,
    create_tracer,
    util,
)
from opentelemetry.test.mock_textmap import (
    MockTextMapPropagator,
    NOOPTextMapPropagator,
)


class TestShim(TestCase):
    # pylint: disable=too-many-public-methods

    def setUp(self):
        """Create an OpenTelemetry tracer and a shim before every test case."""
        trace.set_tracer_provider(TracerProvider())
        self.shim = create_tracer(trace.get_tracer_provider())

    @classmethod
    def setUpClass(cls):
        # Save current propagator to be restored on teardown.
        cls._previous_propagator = get_global_textmap()

        # Set mock propagator for testing.
        set_global_textmap(MockTextMapPropagator())

    @classmethod
    def tearDownClass(cls):
        # Restore previous propagator.
        set_global_textmap(cls._previous_propagator)

    def test_shim_type(self):
        # Verify shim is an OpenTracing tracer.
        self.assertIsInstance(self.shim, opentracing.Tracer)

    def test_start_active_span(self):
        """Test span creation and activation using `start_active_span()`."""

        with self.shim.start_active_span("TestSpan0") as scope:
            # Verify correct type of Scope and Span objects.
            self.assertIsInstance(scope, opentracing.Scope)
            self.assertIsInstance(scope.span, opentracing.Span)

            # Verify span is started.
            self.assertIsNotNone(scope.span.unwrap().start_time)

            # Verify span is active.
            self.assertEqual(
                self.shim.active_span.context.unwrap(),
                scope.span.context.unwrap(),
            )
            # TODO: We can't check for equality of self.shim.active_span and
            # scope.span because the same OpenTelemetry span is returned inside
            # different SpanShim objects. A possible solution is described
            # here:
            # https://github.com/open-telemetry/opentelemetry-python/issues/161#issuecomment-534136274

        # Verify span has ended.
        self.assertIsNotNone(scope.span.unwrap().end_time)

        # Verify no span is active.
        self.assertIsNone(self.shim.active_span)

    def test_start_span(self):
        """Test span creation using `start_span()`."""

        with self.shim.start_span("TestSpan1") as span:
            # Verify correct type of Span object.
            self.assertIsInstance(span, opentracing.Span)

            # Verify span is started.
            self.assertIsNotNone(span.unwrap().start_time)

            # Verify `start_span()` does NOT make the span active.
            self.assertIsNone(self.shim.active_span)

        # Verify span has ended.
        self.assertIsNotNone(span.unwrap().end_time)

    def test_start_span_no_contextmanager(self):
        """Test `start_span()` without a `with` statement."""

        span = self.shim.start_span("TestSpan2")

        # Verify span is started.
        self.assertIsNotNone(span.unwrap().start_time)

        # Verify `start_span()` does NOT make the span active.
        self.assertIsNone(self.shim.active_span)

        span.finish()

    def test_explicit_span_finish(self):
        """Test `finish()` method on `Span` objects."""

        span = self.shim.start_span("TestSpan3")

        # Verify span hasn't ended.
        self.assertIsNone(span.unwrap().end_time)

        span.finish()

        # Verify span has ended.
        self.assertIsNotNone(span.unwrap().end_time)

    def test_explicit_start_time(self):
        """Test `start_time` argument."""

        now = time.time()
        with self.shim.start_active_span("TestSpan4", start_time=now) as scope:
            result = util.time_seconds_from_ns(scope.span.unwrap().start_time)
            # Tolerate inaccuracies of less than a microsecond. See Note:
            # https://open-telemetry.github.io/opentelemetry-python/opentelemetry.shim.opentracing_shim.html
            # TODO: This seems to work consistently, but we should find out the
            # biggest possible loss of precision.
            self.assertAlmostEqual(result, now, places=6)

    def test_explicit_end_time(self):
        """Test `end_time` argument of `finish()` method."""

        span = self.shim.start_span("TestSpan5")
        now = time.time()
        span.finish(now)

        end_time = util.time_seconds_from_ns(span.unwrap().end_time)
        # Tolerate inaccuracies of less than a microsecond. See Note:
        # https://open-telemetry.github.io/opentelemetry-python/opentelemetry.shim.opentracing_shim.html
        # TODO: This seems to work consistently, but we should find out the
        # biggest possible loss of precision.
        self.assertAlmostEqual(end_time, now, places=6)

    def test_explicit_span_activation(self):
        """Test manual activation and deactivation of a span."""

        span = self.shim.start_span("TestSpan6")

        # Verify no span is currently active.
        self.assertIsNone(self.shim.active_span)

        with self.shim.scope_manager.activate(
            span, finish_on_close=True
        ) as scope:
            # Verify span is active.
            self.assertEqual(
                self.shim.active_span.context.unwrap(),
                scope.span.context.unwrap(),
            )

        # Verify no span is active.
        self.assertIsNone(self.shim.active_span)

    def test_start_active_span_finish_on_close(self):
        """Test `finish_on_close` argument of `start_active_span()`."""

        with self.shim.start_active_span(
            "TestSpan7", finish_on_close=True
        ) as scope:
            # Verify span hasn't ended.
            self.assertIsNone(scope.span.unwrap().end_time)

        # Verify span has ended.
        self.assertIsNotNone(scope.span.unwrap().end_time)

        with self.shim.start_active_span(
            "TestSpan8", finish_on_close=False
        ) as scope:
            # Verify span hasn't ended.
            self.assertIsNone(scope.span.unwrap().end_time)

        # Verify span hasn't ended after scope had been closed.
        self.assertIsNone(scope.span.unwrap().end_time)

        scope.span.finish()

    def test_activate_finish_on_close(self):
        """Test `finish_on_close` argument of `activate()`."""

        span = self.shim.start_span("TestSpan9")

        with self.shim.scope_manager.activate(
            span, finish_on_close=True
        ) as scope:
            # Verify span is active.
            self.assertEqual(
                self.shim.active_span.context.unwrap(),
                scope.span.context.unwrap(),
            )

        # Verify span has ended.
        self.assertIsNotNone(span.unwrap().end_time)

        span = self.shim.start_span("TestSpan10")

        with self.shim.scope_manager.activate(
            span, finish_on_close=False
        ) as scope:
            # Verify span is active.
            self.assertEqual(
                self.shim.active_span.context.unwrap(),
                scope.span.context.unwrap(),
            )

        # Verify span hasn't ended.
        self.assertIsNone(span.unwrap().end_time)

        span.finish()

    def test_explicit_scope_close(self):
        """Test `close()` method on `ScopeShim`."""

        with self.shim.start_active_span("ParentSpan") as parent:
            # Verify parent span is active.
            self.assertEqual(
                self.shim.active_span.context.unwrap(),
                parent.span.context.unwrap(),
            )

            child = self.shim.start_active_span("ChildSpan")

            # Verify child span is active.
            self.assertEqual(
                self.shim.active_span.context.unwrap(),
                child.span.context.unwrap(),
            )

            # Verify child span hasn't ended.
            self.assertIsNone(child.span.unwrap().end_time)

            child.close()

            # Verify child span has ended.
            self.assertIsNotNone(child.span.unwrap().end_time)

            # Verify parent span becomes active again.
            self.assertEqual(
                self.shim.active_span.context.unwrap(),
                parent.span.context.unwrap(),
            )

    def test_parent_child_implicit(self):
        """Test parent-child relationship and activation/deactivation of spans
        without specifying the parent span upon creation.
        """

        with self.shim.start_active_span("ParentSpan") as parent:
            # Verify parent span is the active span.
            self.assertEqual(
                self.shim.active_span.context.unwrap(),
                parent.span.context.unwrap(),
            )

            with self.shim.start_active_span("ChildSpan") as child:
                # Verify child span is the active span.
                self.assertEqual(
                    self.shim.active_span.context.unwrap(),
                    child.span.context.unwrap(),
                )

                # Verify parent-child relationship.
                parent_trace_id = (
                    parent.span.unwrap().get_span_context().trace_id
                )
                child_trace_id = (
                    child.span.unwrap().get_span_context().trace_id
                )

                self.assertEqual(parent_trace_id, child_trace_id)
                self.assertEqual(
                    child.span.unwrap().parent,
                    parent.span.unwrap().get_span_context(),
                )

            # Verify parent span becomes the active span again.
            self.assertEqual(
                self.shim.active_span.context.unwrap(),
                parent.span.context.unwrap(),
                # TODO: Check equality of the spans themselves rather than
                # their context once the SpanShim reconstruction problem has
                # been addressed (see previous TODO).
            )

        # Verify there is no active span.
        self.assertIsNone(self.shim.active_span)

    def test_parent_child_explicit_span(self):
        """Test parent-child relationship of spans when specifying a `Span`
        object as a parent upon creation.
        """

        with self.shim.start_span("ParentSpan") as parent:
            with self.shim.start_active_span(
                "ChildSpan", child_of=parent
            ) as child:
                parent_trace_id = parent.unwrap().get_span_context().trace_id
                child_trace_id = (
                    child.span.unwrap().get_span_context().trace_id
                )

                self.assertEqual(child_trace_id, parent_trace_id)
                self.assertEqual(
                    child.span.unwrap().parent,
                    parent.unwrap().get_span_context(),
                )

        with self.shim.start_span("ParentSpan") as parent:
            child = self.shim.start_span("ChildSpan", child_of=parent)

            parent_trace_id = parent.unwrap().get_span_context().trace_id
            child_trace_id = child.unwrap().get_span_context().trace_id

            self.assertEqual(child_trace_id, parent_trace_id)
            self.assertEqual(
                child.unwrap().parent, parent.unwrap().get_span_context()
            )

            child.finish()

    def test_parent_child_explicit_span_context(self):
        """Test parent-child relationship of spans when specifying a
        `SpanContext` object as a parent upon creation.
        """

        with self.shim.start_span("ParentSpan") as parent:
            with self.shim.start_active_span(
                "ChildSpan", child_of=parent.context
            ) as child:
                parent_trace_id = parent.unwrap().get_span_context().trace_id
                child_trace_id = (
                    child.span.unwrap().get_span_context().trace_id
                )

                self.assertEqual(child_trace_id, parent_trace_id)
                self.assertEqual(
                    child.span.unwrap().parent, parent.context.unwrap()
                )

        with self.shim.start_span("ParentSpan") as parent:
            with self.shim.start_span(
                "SpanWithContextParent", child_of=parent.context
            ) as child:
                parent_trace_id = parent.unwrap().get_span_context().trace_id
                child_trace_id = child.unwrap().get_span_context().trace_id

                self.assertEqual(child_trace_id, parent_trace_id)
                self.assertEqual(
                    child.unwrap().parent, parent.context.unwrap()
                )

    def test_references(self):
        """Test span creation using the `references` argument."""

        with self.shim.start_span("ParentSpan") as parent:
            ref = opentracing.child_of(parent.context)

            with self.shim.start_active_span(
                "ChildSpan", references=[ref]
            ) as child:
                self.assertEqual(
                    child.span.unwrap().links[0].context,
                    parent.context.unwrap(),
                )

    def test_follows_from_references(self):
        """Test span creation using the `references` argument with a follows from relationship."""

        with self.shim.start_span("ParentSpan") as parent:
            ref = opentracing.follows_from(parent.context)

        with self.shim.start_active_span(
            "FollowingSpan", references=[ref]
        ) as child:
            self.assertEqual(
                child.span.unwrap().links[0].context,
                parent.context.unwrap(),
            )
            self.assertEqual(
                child.span.unwrap().parent,
                parent.context.unwrap(),
            )

    def test_set_operation_name(self):
        """Test `set_operation_name()` method."""

        with self.shim.start_active_span("TestName") as scope:
            self.assertEqual(scope.span.unwrap().name, "TestName")

            scope.span.set_operation_name("NewName")
            self.assertEqual(scope.span.unwrap().name, "NewName")

    def test_tags(self):
        """Test tags behavior using the `tags` argument and the `set_tags()`
        method.
        """

        tags = {"foo": "bar"}
        with self.shim.start_active_span("TestSetTag", tags=tags) as scope:
            scope.span.set_tag("baz", "qux")

            self.assertEqual(scope.span.unwrap().attributes["foo"], "bar")
            self.assertEqual(scope.span.unwrap().attributes["baz"], "qux")

    def test_span_tracer(self):
        """Test the `tracer` property on `Span` objects."""

        with self.shim.start_active_span("TestSpan11") as scope:
            self.assertEqual(scope.span.tracer, self.shim)

    def test_log_kv(self):
        """Test the `log_kv()` method on `Span` objects."""

        with self.shim.start_span("TestSpan12") as span:
            span.log_kv({"foo": "bar"})
            self.assertEqual(span.unwrap().events[0].attributes["foo"], "bar")
            # Verify timestamp was generated automatically.
            self.assertIsNotNone(span.unwrap().events[0].timestamp)

            # Test explicit timestamp.
            now = time.time()
            span.log_kv({"foo": "bar"}, now)
            result = util.time_seconds_from_ns(
                span.unwrap().events[1].timestamp
            )
            self.assertEqual(span.unwrap().events[1].attributes["foo"], "bar")
            # Tolerate inaccuracies of less than a microsecond. See Note:
            # https://open-telemetry.github.io/opentelemetry-python/shim/opentracing_shim/opentracing_shim.html
            # TODO: This seems to work consistently, but we should find out the
            # biggest possible loss of precision.
            self.assertAlmostEqual(result, now, places=6)

    def test_log(self):
        """Test the deprecated `log` method on `Span` objects."""

        with self.shim.start_span("TestSpan13") as span:
            with self.assertWarns(DeprecationWarning):
                span.log(event="foo", payload="bar")

        self.assertEqual(span.unwrap().events[0].attributes["event"], "foo")
        self.assertEqual(span.unwrap().events[0].attributes["payload"], "bar")
        self.assertIsNotNone(span.unwrap().events[0].timestamp)

    def test_log_event(self):
        """Test the deprecated `log_event` method on `Span` objects."""

        with self.shim.start_span("TestSpan14") as span:
            with self.assertWarns(DeprecationWarning):
                span.log_event("foo", "bar")

        self.assertEqual(span.unwrap().events[0].attributes["event"], "foo")
        self.assertEqual(span.unwrap().events[0].attributes["payload"], "bar")
        self.assertIsNotNone(span.unwrap().events[0].timestamp)

    def test_span_context(self):
        """Test construction of `SpanContextShim` objects."""

        otel_context = trace.SpanContext(1234, 5678, is_remote=False)
        context = SpanContextShim(otel_context)

        self.assertIsInstance(context, opentracing.SpanContext)
        self.assertEqual(context.unwrap().trace_id, 1234)
        self.assertEqual(context.unwrap().span_id, 5678)

    def test_span_on_error(self):
        """Verify error tag and logs are created on span when an exception is
        raised.
        """

        # Raise an exception while a span is active.
        with self.assertRaises(Exception) as exc_ctx:
            with self.shim.start_active_span("TestName") as scope:
                # pylint: disable=broad-exception-raised
                raise Exception("bad thing")

        ex = exc_ctx.exception
        expected_stack = "".join(
            traceback.format_exception(type(ex), value=ex, tb=ex.__traceback__)
        )
        # Verify exception details have been added to span.
        exc_event = scope.span.unwrap().events[0]

        self.assertEqual(exc_event.name, "exception")
        self.assertEqual(
            exc_event.attributes["exception.message"], "bad thing"
        )
        self.assertEqual(
            exc_event.attributes["exception.type"], Exception.__name__
        )
        # cannot get the whole stacktrace so just assert exception part is contained
        self.assertIn(
            expected_stack, exc_event.attributes["exception.stacktrace"]
        )

    def test_inject_http_headers(self):
        """Test `inject()` method for Format.HTTP_HEADERS."""

        otel_context = trace.SpanContext(
            trace_id=1220, span_id=7478, is_remote=False
        )
        context = SpanContextShim(otel_context)

        headers = {}
        self.shim.inject(context, opentracing.Format.HTTP_HEADERS, headers)
        self.assertEqual(
            headers[MockTextMapPropagator.TRACE_ID_KEY], str(1220)
        )
        self.assertEqual(headers[MockTextMapPropagator.SPAN_ID_KEY], str(7478))

    def test_inject_text_map(self):
        """Test `inject()` method for Format.TEXT_MAP."""

        otel_context = trace.SpanContext(
            trace_id=1220, span_id=7478, is_remote=False
        )
        context = SpanContextShim(otel_context)

        # Verify Format.TEXT_MAP
        text_map = {}
        self.shim.inject(context, opentracing.Format.TEXT_MAP, text_map)
        self.assertEqual(
            text_map[MockTextMapPropagator.TRACE_ID_KEY], str(1220)
        )
        self.assertEqual(
            text_map[MockTextMapPropagator.SPAN_ID_KEY], str(7478)
        )

    def test_inject_binary(self):
        """Test `inject()` method for Format.BINARY."""

        otel_context = trace.SpanContext(
            trace_id=1220, span_id=7478, is_remote=False
        )
        context = SpanContextShim(otel_context)

        # Verify exception for non supported binary format.
        with self.assertRaises(opentracing.UnsupportedFormatException):
            self.shim.inject(context, opentracing.Format.BINARY, bytearray())

    def test_extract_http_headers(self):
        """Test `extract()` method for Format.HTTP_HEADERS."""

        carrier = {
            MockTextMapPropagator.TRACE_ID_KEY: 1220,
            MockTextMapPropagator.SPAN_ID_KEY: 7478,
        }

        ctx = self.shim.extract(opentracing.Format.HTTP_HEADERS, carrier)
        self.assertEqual(ctx.unwrap().trace_id, 1220)
        self.assertEqual(ctx.unwrap().span_id, 7478)

    def test_extract_empty_context_returns_invalid_context(self):
        """In the case where the propagator cannot extract a
        SpanContext, extract should return and invalid span context.
        """
        _old_propagator = get_global_textmap()
        set_global_textmap(NOOPTextMapPropagator())
        try:
            carrier = {}

            ctx = self.shim.extract(opentracing.Format.HTTP_HEADERS, carrier)
            self.assertEqual(ctx.unwrap(), trace.INVALID_SPAN_CONTEXT)
        finally:
            set_global_textmap(_old_propagator)

    def test_extract_text_map(self):
        """Test `extract()` method for Format.TEXT_MAP."""

        carrier = {
            MockTextMapPropagator.TRACE_ID_KEY: 1220,
            MockTextMapPropagator.SPAN_ID_KEY: 7478,
        }

        ctx = self.shim.extract(opentracing.Format.TEXT_MAP, carrier)
        self.assertEqual(ctx.unwrap().trace_id, 1220)
        self.assertEqual(ctx.unwrap().span_id, 7478)

    def test_extract_binary(self):
        """Test `extract()` method for Format.BINARY."""

        # Verify exception for non supported binary format.
        with self.assertRaises(opentracing.UnsupportedFormatException):
            self.shim.extract(opentracing.Format.BINARY, bytearray())

    def test_baggage(self):
        span_context_shim = SpanContextShim(
            trace.SpanContext(1234, 5678, is_remote=False)
        )

        baggage = span_context_shim.baggage

        with self.assertRaises(ValueError):
            baggage[1] = 3

        span_shim = SpanShim(Mock(), span_context_shim, Mock())

        span_shim.set_baggage_item(1, 2)

        self.assertTrue(span_shim.get_baggage_item(1), 2)

    def test_active(self):
        """Test that the active property and start_active_span return the same
        object"""

        # Verify no span is currently active.
        self.assertIsNone(self.shim.active_span)

        with self.shim.start_active_span("TestSpan15") as scope:
            # Verify span is active.
            self.assertEqual(
                self.shim.active_span.context.unwrap(),
                scope.span.context.unwrap(),
            )

            self.assertIs(self.shim.scope_manager.active, scope)

        # Verify no span is active.
        self.assertIsNone(self.shim.active_span)

    def test_mixed_mode(self):
        """Test that span parent-child relationship is kept between
        OpenTelemetry and the OpenTracing shim"""

        span_shim = self.shim.start_span("TestSpan16")

        with self.shim.scope_manager.activate(span_shim, finish_on_close=True):
            with (
                TracerProvider()
                .get_tracer(__name__)
                .start_as_current_span("abc")
            ) as opentelemetry_span:
                self.assertIs(
                    span_shim.unwrap().context,
                    opentelemetry_span.parent,
                )

        with (
            TracerProvider().get_tracer(__name__).start_as_current_span("abc")
        ) as opentelemetry_span:
            with self.shim.start_active_span("TestSpan17") as scope:
                self.assertIs(
                    scope.span.unwrap().parent,
                    opentelemetry_span.context,
                )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum
from typing import Final

from typing_extensions import deprecated

FEATURE_FLAG_CONTEXT_ID: Final = "feature_flag.context.id"
"""
The unique identifier for the flag evaluation context. For example, the targeting key.
"""

FEATURE_FLAG_ERROR_MESSAGE: Final = "feature_flag.error.message"
"""
A message providing more detail about an error that occurred during feature flag evaluation in human-readable form.
"""

FEATURE_FLAG_EVALUATION_ERROR_MESSAGE: Final = (
    "feature_flag.evaluation.error.message"
)
"""
Deprecated: Replaced by `feature_flag.error.message`.
"""

FEATURE_FLAG_EVALUATION_REASON: Final = "feature_flag.evaluation.reason"
"""
Deprecated: Replaced by `feature_flag.result.reason`.
"""

FEATURE_FLAG_KEY: Final = "feature_flag.key"
"""
The lookup key of the feature flag.
"""

FEATURE_FLAG_PROVIDER_NAME: Final = "feature_flag.provider.name"
"""
Identifies the feature flag provider.
"""

FEATURE_FLAG_RESULT_REASON: Final = "feature_flag.result.reason"
"""
The reason code which shows how a feature flag value was determined.
"""

FEATURE_FLAG_RESULT_VALUE: Final = "feature_flag.result.value"
"""
The evaluated value of the feature flag.
Note: With some feature flag providers, feature flag results can be quite large or contain private or sensitive details.
Because of this, `feature_flag.result.variant` is often the preferred attribute if it is available.

It may be desirable to redact or otherwise limit the size and scope of `feature_flag.result.value` if possible.
Because the evaluated flag value is unstructured and may be any type, it is left to the instrumentation author to determine how best to achieve this.
"""

FEATURE_FLAG_RESULT_VARIANT: Final = "feature_flag.result.variant"
"""
A semantic identifier for an evaluated flag value.
Note: A semantic identifier, commonly referred to as a variant, provides a means
for referring to a value without including the value itself. This can
provide additional context for understanding the meaning behind a value.
For example, the variant `red` maybe be used for the value `#c05543`.
"""

FEATURE_FLAG_SET_ID: Final = "feature_flag.set.id"
"""
The identifier of the [flag set](https://openfeature.dev/specification/glossary/#flag-set) to which the feature flag belongs.
"""

FEATURE_FLAG_VARIANT: Final = "feature_flag.variant"
"""
Deprecated: Replaced by `feature_flag.result.variant`.
"""

FEATURE_FLAG_VERSION: Final = "feature_flag.version"
"""
The version of the ruleset used during the evaluation. This may be any stable value which uniquely identifies the ruleset.
"""


@deprecated(
    "The attribute feature_flag.evaluation.reason is deprecated - Replaced by `feature_flag.result.reason`"
)
class FeatureFlagEvaluationReasonValues(Enum):
    STATIC = "static"
    """The resolved value is static (no dynamic evaluation)."""
    DEFAULT = "default"
    """The resolved value fell back to a pre-configured value (no dynamic evaluation occurred or dynamic evaluation yielded no result)."""
    TARGETING_MATCH = "targeting_match"
    """The resolved value was the result of a dynamic evaluation, such as a rule or specific user-targeting."""
    SPLIT = "split"
    """The resolved value was the result of pseudorandom assignment."""
    CACHED = "cached"
    """The resolved value was retrieved from cache."""
    DISABLED = "disabled"
    """The resolved value was the result of the flag being disabled in the management system."""
    UNKNOWN = "unknown"
    """The reason for the resolved value could not be determined."""
    STALE = "stale"
    """The resolved value is non-authoritative or possibly out of date."""
    ERROR = "error"
    """The resolved value was the result of an error."""


class FeatureFlagResultReasonValues(Enum):
    STATIC = "static"
    """The resolved value is static (no dynamic evaluation)."""
    DEFAULT = "default"
    """The resolved value fell back to a pre-configured value (no dynamic evaluation occurred or dynamic evaluation yielded no result)."""
    TARGETING_MATCH = "targeting_match"
    """The resolved value was the result of a dynamic evaluation, such as a rule or specific user-targeting."""
    SPLIT = "split"
    """The resolved value was the result of pseudorandom assignment."""
    CACHED = "cached"
    """The resolved value was retrieved from cache."""
    DISABLED = "disabled"
    """The resolved value was the result of the flag being disabled in the management system."""
    UNKNOWN = "unknown"
    """The reason for the resolved value could not be determined."""
    STALE = "stale"
    """The resolved value is non-authoritative or possibly out of date."""
    ERROR = "error"
    """The resolved value was the result of an error."""



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import unittest

from opentelemetry import trace


class RecordingSpan(trace.Span):
    def __init__(self, context: trace.SpanContext) -> None:
        self._context = context

    def get_span_context(self) -> trace.SpanContext:
        return self._context

    def is_recording(self) -> bool:
        return True

    def end(self, end_time=None) -> None:
        pass

    def set_attributes(self, attributes) -> None:
        pass

    def set_attribute(self, key, value) -> None:
        pass

    def add_event(
        self,
        name: str,
        attributes=None,
        timestamp=None,
    ) -> None:
        pass

    def add_link(
        self,
        context,
        attributes=None,
    ) -> None:
        pass

    def update_name(self, name) -> None:
        pass

    def set_status(
        self,
        status,
        description=None,
    ) -> None:
        pass

    def record_exception(
        self,
        exception,
        attributes=None,
        timestamp=None,
        escaped=False,
    ) -> None:
        pass

    def __repr__(self) -> str:
        return f"RecordingSpan({self._context!r})"


class TestAPIOnlyImplementation(unittest.TestCase):
    """
    This test is in place to ensure the API is returning values that
    are valid. The same tests have been added to the SDK with
    different expected results. See issue for more details:
    https://github.com/open-telemetry/opentelemetry-python/issues/142
    """

    # TRACER

    def test_tracer(self):
        with self.assertRaises(TypeError):
            # pylint: disable=abstract-class-instantiated
            trace.TracerProvider()  # type:ignore

    def test_default_tracer(self):
        tracer_provider = trace.NoOpTracerProvider()
        tracer = tracer_provider.get_tracer(__name__)
        with tracer.start_span("test") as span:
            self.assertFalse(span.get_span_context().is_valid)
            self.assertIs(span.is_recording(), False)
            with tracer.start_span("test2") as span2:
                self.assertFalse(span2.get_span_context().is_valid)
                self.assertIs(span2.is_recording(), False)

    def test_default_tracer_context_propagation_recording_span(self):
        tracer_provider = trace.NoOpTracerProvider()
        tracer = tracer_provider.get_tracer(__name__)
        span_context = trace.SpanContext(
            2604504634922341076776623263868986797,
            5213367945872657620,
            False,
            trace.TraceFlags(0x01),
        )
        ctx = trace.set_span_in_context(RecordingSpan(context=span_context))
        with tracer.start_span("test", context=ctx) as span:
            self.assertTrue(span.get_span_context().is_valid)
            self.assertEqual(span.get_span_context(), span_context)
            self.assertIs(span.is_recording(), False)

    def test_default_tracer_context_propagation_non_recording_span(self):
        tracer_provider = trace.NoOpTracerProvider()
        tracer = tracer_provider.get_tracer(__name__)
        ctx = trace.set_span_in_context(trace.INVALID_SPAN)
        with tracer.start_span("test", context=ctx) as span:
            self.assertFalse(span.get_span_context().is_valid)
            self.assertIs(span, trace.INVALID_SPAN)

    def test_default_tracer_context_propagation_with_invalid_context(self):
        tracer_provider = trace.NoOpTracerProvider()
        tracer = tracer_provider.get_tracer(__name__)
        ctx = trace.set_span_in_context(
            RecordingSpan(context="invalid_context")  # type: ignore[reportArgumentType]
        )
        with tracer.start_span("test", context=ctx) as span:
            self.assertFalse(span.get_span_context().is_valid)
            self.assertIs(span, trace.INVALID_SPAN)

    def test_span(self):
        with self.assertRaises(TypeError):
            # pylint: disable=abstract-class-instantiated
            trace.Span()  # type:ignore

    def test_default_span(self):
        span = trace.NonRecordingSpan(trace.INVALID_SPAN_CONTEXT)
        self.assertEqual(span.get_span_context(), trace.INVALID_SPAN_CONTEXT)
        self.assertIs(span.is_recording(), False)
